#include <windows.h>
#include <stdio.h>
#include "resource.h"
#include "LALANIO.h"

BOOL CALLBACK DlgProc(HWND, UINT, WPARAM, LPARAM);
BOOL connect(HWND hWnd, hLANIO *lanio);
void cycle(HWND hWnd, hLANIO lanio);
BOOL start(HWND hWnd, hLANIO lanio);
void stop(hLANIO lanio);
void callback(HWND hWnd, int *input);
BOOL connected = FALSE;
BOOL inputing = FALSE;
hLANIO lanio = 0;
const char *timestr[] = {
	"0.05sec", "0.1sec", "0.2sec", "0.5sec", "1sec", "2sec", "5sec",
	"10sec", "20sec", "30sec", "1min", "2min", "5min", "10min", "" };

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
	DialogBox(NULL, MAKEINTRESOURCE(IDD_CYCLIC), NULL, (DLGPROC)DlgProc);
	return 0;
}

BOOL CALLBACK DlgProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	int i;
    switch( msg ){
	case WM_INITDIALOG:
		for( i = 0; timestr[i][0] != NULL; i ++ ){
			SendMessage(GetDlgItem(hWnd, IDC_CYCLE), CB_ADDSTRING, 0, (LPARAM)timestr[i]);
		}
		LALanioInit();
		return TRUE;
	case WM_CLOSE:
		LALanioEnd();
		EndDialog(hWnd, IDOK);
		return TRUE;
	case WM_COMMAND:
		switch( LOWORD(wParam) ){
		case IDCANCEL:
			LALanioEnd();
			EndDialog(hWnd, IDCANCEL);
			return TRUE;
		case IDC_CONNECT:
			if( IsDlgButtonChecked(hWnd, IDC_CONNECT) == BST_CHECKED ){
				// connecting
				if( connect(hWnd, &lanio) == TRUE ){
					connected = TRUE;
				} else {
					CheckDlgButton(hWnd, IDC_CONNECT, BST_UNCHECKED);
				}
				return TRUE;
			} else {
				// disconnecting
				LALanioClose(lanio);
				connected = FALSE;
			}
			return TRUE;
		case IDC_CYCLE_SET:
			cycle(hWnd, lanio);
			return TRUE;
		case IDC_START:
			if( IsDlgButtonChecked(hWnd, IDC_START) == BST_CHECKED ){
				// start
				if( start(hWnd, lanio) == TRUE ){
					inputing = TRUE;
				} else {
					CheckDlgButton(hWnd, IDC_START, BST_UNCHECKED);
				}
				return TRUE;
			} else {
				// end
				stop(lanio);
				inputing = FALSE;
			}
			return TRUE;
		}
	}
	return FALSE;
}

BOOL connect(HWND hWnd, hLANIO *lanio)
{
	char ipaddress[256];
	if( GetDlgItemText(hWnd, IDC_IPADDRESS, ipaddress, 256) == 0 ){
		return FALSE;
	}
	*lanio = LALanioConnectDirect(ipaddress, 2000);
	if( *lanio == -1 ){
		MessageBox(hWnd,"Connect error",NULL,IDOK);
		return FALSE;
	}
	return TRUE;
}

void cycle(HWND hWnd, hLANIO lanio)
{
	int sel;
	sel = SendMessage(GetDlgItem(hWnd, IDC_CYCLE), CB_GETCURSEL, 0, 0);
	if( sel != LB_ERR ){
		int err;
		err = LALanioSetCyclicTransmit(lanio, sel);
		if( err == 0 ){
			MessageBox(hWnd,"Cycle set error",NULL,IDOK);
		}
	}
}

BOOL start(HWND hWnd, hLANIO lanio)
{
	int err;
	err = LALanioStartCyclicTransmit(lanio, callback, hWnd);
	if( err == 0 ){
		MessageBox(hWnd,"Start error",NULL,IDOK);
		return FALSE;
	}
	return TRUE;
}

void stop(hLANIO lanio)
{
	LALanioStopCyclicTransmit(lanio);
}

void callback(HWND hWnd, int *input)
{
	if( input[0] == 0x80000000 ){
		SetDlgItemText(hWnd, IDC_AI1, "error");
		SetDlgItemText(hWnd, IDC_AI2, "error");
		SetDlgItemText(hWnd, IDC_AI3, "error");
	} else {
		SetDlgItemInt(hWnd, IDC_AI1, input[0], TRUE);
		SetDlgItemInt(hWnd, IDC_AI2, input[1], TRUE);
		SetDlgItemInt(hWnd, IDC_AI3, input[2], TRUE);
	}
}
